#!/usr/bin/perl

use strict;

use Cwd  qw(abs_path);
use File::Basename;
use CGI;
use File::Glob ':glob';

# Untaint environment variables
#$ENV{'PATH'}   = "/bin:/usr/bin:/usr/local/bin:/sbin:/usr/sbin:/sw/bin:/sw/sbin";
$ENV{'PATH'} = untaint($ENV{'PATH'});
delete @ENV{qw(IFS CDPATH ENV BASH_ENV)};

my $q = new CGI;


my %CONFIG;

$CONFIG{'EXECDIR'} = '/Users/seb/Sites/webapps/medeller'; # where the web front-end sits
$CONFIG{'APPDIR'} = '/Users/seb/webserver/apps/medeller'; # where the back-end app sits
$CONFIG{'JOBSDIR'} = '/tmp/www-medeller'; # where user uploads (i.e. input files) go
$CONFIG{'CACHEDIR'} = untaint($CONFIG{'EXECDIR'}.'/cache'); # where web-accessible output files go

$CONFIG{'APPNAME'} = get_input('app', qr/^MEDELLER$|^iMembrane$|^Completionist$|^iMemDB$/i, 'Completionist');
#$CONFIG{'APPNAME'} = get_input('app', qr/^Completionist$/i, 'Completionist');

# Fix case of APPNAME
#
if ($CONFIG{'APPNAME'} =~ /^iMembrane$/i)
{
  $CONFIG{'APPNAME'} = 'iMembrane';
  $CONFIG{'APPCOMMAND'}    = 'automodel';
}
elsif ($CONFIG{'APPNAME'} =~ /^MEDELLER$/i)
{
  $CONFIG{'APPNAME'} = 'MEDELLER';
  $CONFIG{'APPCOMMAND'}    = 'automodel';
}
elsif ($CONFIG{'APPNAME'} =~ /^Completionist$/i)
{
  $CONFIG{'APPNAME'} = 'Completionist';
  $CONFIG{'APPCOMMAND'}    = 'completionist';
}
elsif ($CONFIG{'APPNAME'} =~ /^iMemDB$/i)
{
  $CONFIG{'APPNAME'} = 'iMemDB';
  $CONFIG{'APPCOMMAND'}    = 'echo';
}

$CONFIG{'STYLESHEET'}    = lc($CONFIG{'APPNAME'}).'.css';

# Paths to the Command Line Utilities
#$CONFIG{'LIBDIR'}        = untaint($CONFIG{'APPDIR'}.'/lib-python');
$CONFIG{'BINDIR'}        = untaint($CONFIG{'APPDIR'}.'/bin');
$ENV{'PATH'} = untaint($CONFIG{'BINDIR'}.':'.$ENV{'PATH'});

# Path to local version of the Protein Data Bank
$CONFIG{'PDBDIR'}        = untaint($CONFIG{'APPDIR'}.'/pdb');

# Maximum consecutively running jobs in the Web GUI
$CONFIG{'PARALLELJOBS'}  = 1;


# Filenames
$CONFIG{'OPTIONSFILE'}   = 'opts.txt';

# URLs
$CONFIG{'SERVERURL'}     = './';
$CONFIG{'HOMEURL'}       = $CONFIG{'SERVERURL'}.'home.pl';
$CONFIG{'HELPURL'}       = $CONFIG{'SERVERURL'}.'help.pl?app='.$CONFIG{'APPNAME'};
$CONFIG{'SUBMITURL'}     = $CONFIG{'SERVERURL'}.'submit.pl';
$CONFIG{'QUEUEURL'}      = $CONFIG{'SERVERURL'}.'queue.pl';
$CONFIG{'VIEWERURL'}     = $CONFIG{'SERVERURL'}.'viewer.pl';
$CONFIG{'ARCHIVEURL'}    = $CONFIG{'SERVERURL'}.'archive.pl';
$CONFIG{'CACHERELATIVEURL'} = 'cache';
$CONFIG{'CACHEURL'}      = $CONFIG{'SERVERURL'}.$CONFIG{'CACHERELATIVEURL'};


my %subtitles = ('MEDELLER', 'Homology-Based Coordinate Generation for Membrane Proteins', 'iMembrane', 'Homology-Based Insertion of Proteins into the Membrane', 'Completionist', 'Model Completion for Membrane, Soluble and Ig-like Proteins');

$CONFIG{'HTML_MINIMAL_HEADER'}='
<h1><a href="'.internallink('home').'">'.html_pretty($CONFIG{'APPNAME'}).'</a> <span class="minititle">'.$subtitles{$CONFIG{'APPNAME'}}.'</span></h1>
';

$CONFIG{'HTML_HEADER'}=qq(
<div align="center"><table><tr><td> <!-- begin CENTER -->
$CONFIG{'HTML_MINIMAL_HEADER'}
<div class="mainmargin">
);


my $currentyear = get_current_year();
$CONFIG{'HTML_MINIMAL_FOOTER'}='
<div class="footer">
<div class="footermenu"><a href="'.internallink('home').'">'.html_pretty('Submit new query').'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</a></div>
<a target="_blank" href="http://www.stats.ox.ac.uk/proteins/">Protein Informatics Group</a> &nbsp; | &nbsp; &copy; <a target="_blank" href="http://www.stats.ox.ac.uk/~kelm/">Sebastian Kelm</a>, Jiye Shi, <a target="_blank" href="http://www.stats.ox.ac.uk/~deane/">Charlotte M. Deane</a>, 2008-'.$currentyear.'
</div>
';

$CONFIG{'HTML_FOOTER'}=qq(
</div>
$CONFIG{'HTML_MINIMAL_FOOTER'}
</td></tr></table></div> <!-- end CENTER -->
);




#unless (-d $CONFIG{'JOBSDIR'})
#{
#  server_error("Jobs directory does not exist: ".$CONFIG{'JOBSDIR'}."\n");
#}
unless (-d $CONFIG{'CACHEDIR'})
{
  server_error("Cache directory does not exist: ".$CONFIG{'CACHEDIR'}."\n");
}



########################################################################



sub get_config
{
  my ($filename) = @_;
  unless($filename)
  {
    return \%CONFIG;
  }
  open(my $f, "$filename");
  while(my $line = <$f>)
  {
    if ($line =~ /^\s*$|^\s*#/)
    {
      # Ignore empty lines and comments
      next;
    }
    chomp $line;
    my ($key, $value) = split($line, 2);
    $CONFIG{$key} = $value;
  }
  close($f);
  return \%CONFIG;
}


sub get_input
{
  my ($name, $re, $default) = @_;
  my $param = $q->param($name);
  if (!defined($param))
  {
    $param = $default;
  }
  if (!defined($param) or ($param !~ m/($re)/))
  {
    if (!defined($param))
    {
      $param = '<undefined>';
    }
    $param =~ s/&/&amp;/g;
    $param =~ s/</&lt;/g;
    $param =~ s/>/&gt;/g;
    finish("Illegal value for parameter '$name': $param\n");
  }
  $param = $1;
  return $param;
}


sub write_file
{
  my ($filename, $text) = @_;
  unless(defined($filename))
  {
    die "Cannot write to file with undefined name.";
  }
  unless(defined($text))
  {
    $text = '';
  }
  open(my $file, ">$filename") or die "Cannot write file $filename";
  chomp $text;
  print $file "$text\n";
  close($file);
}


# Check status of a job
sub job_getstatus
{
  my ($JOB) = @_;
  my $OUTDIR;
  $JOB =~ s%^.*?(\w+)/?$%$1%;
  $OUTDIR = $CONFIG{'CACHEDIR'}."/$JOB";
  $JOB    = $CONFIG{'JOBSDIR'}."/$JOB";

  my @statuslist;
  push @statuslist, glob("$JOB/status.*");
  push @statuslist, glob("$OUTDIR/status.*");
  
  my @status_indeces;
  my $laststat;
  my $lasttime = 0;
  
  for (my $i=0; $i<@statuslist; $i++)
  {
    my $s = $statuslist[$i];
    my @stat  = stat($s);
    my $mtime = $stat[9];
    if ($mtime >= $lasttime)
    {
      $s =~ /(\w+)$/;
      my $sname = $1;
      if ($sname eq 'crashed' or $sname eq 'done')
      {
        $laststat = $sname;
        $lasttime = $mtime;
        splice @status_indeces; # empty the list
        push @status_indeces, $i;
        last;
      }

      if ($mtime == $lasttime)
      {
        $laststat .= "|$sname";
        push @status_indeces, $i;
      }
      else
      {
        $laststat = $sname;
        $lasttime = $mtime;
        splice @status_indeces; # empty the list
        push @status_indeces, $i;
      }
    }
  }
  
  my $message = '';
  for my $i (@status_indeces)
  {
    local $/ = undef;
    open(my $f, $statuslist[$i]) or server_error("Could not open job status file for job: $JOB (${statuslist[$i]})");
    $message .= <$f>;
    close($f);
  }
  
  return $laststat, $lasttime, $message;
}


sub server_error
{
  my ($message) = @_;
  print $q->header('text/html', '500 Internal Server Error');
  print $q->start_html('Internal Server Error');
  print $q->pre($message);
  print $q->end_html();
  exit 0;
}


sub untaint
{
  my ($v) = @_;
  $v =~ /^(.*)$/;
  return $1;
}


sub html_pretty
{
  my ($text, $douc) = @_;
  if ($douc)
  {
    $text = ucfirst $text;
  }
  my $num_letters = 1;
  if ($CONFIG{'APPNAME'} eq 'MEDELLER')
  {
    $num_letters = 2;
  }
  return '<span class="headingstart">'.substr($text, 0, $num_letters).'</span><span class="headingend">'.substr($text, $num_letters).'</span>';
}


sub get_current_year
{
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time());
  $year += 1900;
  return $year;
}


sub selflink
{
  my ($options, $filename) = @_;
  if ($options)
  {
    $options = "&amp;$options";
  }
  else
  {
    $options = '';
  }
  if ($filename)
  {
    $filename = "/$filename";
  }
  else
  {
    $filename = '';
  }
  
  my $result_id = $q->param('result_id');
  if (defined($result_id))
  {
    return $q->url() . "${filename}?app=$CONFIG{'APPNAME'}&amp;result_id=$result_id$options";
  }
  
  return return $q->url() . "${filename}?app=$CONFIG{'APPNAME'}&amp;$options";
}


sub internallink
{
  my ($section, $app) = @_;
  unless ($app)
  {
    $app = $CONFIG{'APPNAME'};
  }
  return "$CONFIG{'SERVERURL'}$section.pl?app=$app";
}


1;
