#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

import sys
import os
import subprocess


from prosci.util.command import Command


class SuperpositionFailed(RuntimeError):
  pass


def superpose_files(freference, fmoving, foutput, sel_reference="", sel_moving="", timeout=sys.maxint):
  if sel_moving or sel_reference:
    command = "superpose '%s' -s '%s' '%s' -s '%s' '%s'"%(fmoving, sel_moving, freference, sel_reference, foutput)
  else:
    command = "superpose '%s' '%s' '%s'"%(fmoving, freference, foutput)

  print command
  c = Command(command)
  c.run(timeout, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
  
  if c.timedout:
    raise SuperpositionFailed("Superpose job timed out")
  elif c.process.returncode != 0:
    raise SuperpositionFailed("Superpose crashed... or something")
  
  return c.out, c.err


def superpose_parse_output(txt):
  lines = txt.replace("\r\n", "\n").replace("\r", "\n").splitlines()
  
  def parse_res(query):
    if query.isspace():
      return None
    
    #sstruc = query[0]
    #match = query[1]
    try:
      chain, res = query[3:8].strip().split(":")
    except ValueError:
      chain = ""
      res = query[3:8].strip()
    
    ires = int(query[8:12].strip())
    inscode = query[12].strip()
    return (chain, res, ires, inscode)
  
  output = []
  for line in lines:
    if not line.startswith("|"):
      continue
    fields = line.rstrip().strip("|").split("|")
    if len(fields) != 3:
      continue
    query, dist, target = fields

    if query.strip() == "Query":
      continue
    
    query_data = parse_res(query)
    
    if dist.isspace():
      dist = None
    else:
      dist = float(dist.strip("<*+=-:.> "))
    
    target_data = parse_res(target)
    
    output.append([target_data, query_data, dist])
  
  return output


def superpose(*args, **kwargs):
    out, err = superpose_files(*args, **kwargs)
    aligned_pairs = superpose_parse_output(out)
    return aligned_pairs, err
